import {
    defaultDegrees,
    defaultDegreesForDrop,
    defaultFallDamageStatus,
    defaultFirstTypeParticleId,
    defaultFirstTypeSoundId,
    defaultHS,
    defaultMaxImpulseCount,
    defaultPushViewDirection,
    defaultSecondTypeParticleColorId,
    defaultSecondTypeParticleId,
    defaultSecondTypeSoundId,
    defaultThirdTypeParticleId,
    defaultTickInterval,
    defaultVS,
    defaultVSForDrop
} from "./config/mainConfig";
import {system, world} from "@minecraft/server";
import {uiManager} from "@minecraft/server-ui";

const dataCache = {};

export function deleteJumpPadSettings(location) {
    const propertyId = getPropertyIdByLocation(location);
    if (world.getDynamicProperty(propertyId) !== undefined) {
        world.setDynamicProperty(propertyId, undefined);
    }
    delete dataCache[propertyId]
}

export function getJumpPadSettings(location) {
    const propertyId = getPropertyIdByLocation(location);

    if (dataCache[propertyId] === undefined) {
        const propertyData = world.getDynamicProperty(propertyId);

        if (propertyData === undefined) {
            const defaultSettings = getDefaultSettings();
            setJumpPadSettings(location, undefined);
            return defaultSettings;
        } else {
            const parsedData = JSON.parse(propertyData);
            dataCache[propertyId] = parsedData;
            return parsedData;
        }
    }

    return dataCache[propertyId];
}

export function setJumpPadSettings(location, settingsObject) {
    let settings
    if (settingsObject) {
        settings = {
            d: settingsObject.d,

            pVD: settingsObject.pVD,

            hS: settingsObject.hS,
            vS: settingsObject.vS,
            mIC: settingsObject.mIC,
            tI: settingsObject.tI,
            vSFD: settingsObject.vSFD,
            dFD: settingsObject.dFD,

            fTP: settingsObject.fTP,
            sTP: settingsObject.sTP,
            tTP: settingsObject.tTP,

            fTS: settingsObject.fTS,
            sTS: settingsObject.sTS,

            sTPC: settingsObject.sTPC,

            fDS: settingsObject.fDS
        };
    } else {
        settings = Object.assign({}, getDefaultSettings());
    }

    const propertyId = getPropertyIdByLocation(location);

    dataCache[propertyId] = settings;
    world.setDynamicProperty(propertyId, JSON.stringify(settings));
}

function getPropertyIdByLocation(location) {
    return `vf_jp:jp_${location.x}_${location.y}_${location.z}`;
}

export function getDefaultSettings() {
    return {
        d: defaultDegrees,

        pVD: defaultPushViewDirection,

        hS: defaultHS,
        vS: defaultVS,
        mIC: defaultMaxImpulseCount,
        tI: defaultTickInterval,
        vSFD: defaultVSForDrop,
        dFD: defaultDegreesForDrop,

        fTP: defaultFirstTypeParticleId,
        sTP: defaultSecondTypeParticleId,
        tTP: defaultThirdTypeParticleId,

        fTS: defaultFirstTypeSoundId,
        sTS: defaultSecondTypeSoundId,

        sTPC: defaultSecondTypeParticleColorId,

        fDS: defaultFallDamageStatus
    };
}

export const trailPropertyId = "vf_jp:tId"
const trailsCache = {};

export function setTrailSettings(player, trailId, colorId = undefined) {
    const property = {trailId: trailId, colorId: colorId};
    player.setDynamicProperty(trailPropertyId, JSON.stringify(property))
    trailsCache[player.id] = property
}

export function getTrailSettings(player) {
    // console.error(JSON.stringify(trailsCache[player.id]));
    if (trailsCache[player.id] === undefined) {
        if (player.getDynamicProperty(trailPropertyId) === undefined) {
            const property = {trailId: undefined, colorId: undefined}
            player.setDynamicProperty(trailPropertyId, JSON.stringify(property))
            trailsCache[player.id] = property
        } else {
            trailsCache[player.id] = JSON.parse(player.getDynamicProperty(trailPropertyId))
        }
    }

    return trailsCache[player.id]
}

const jumppadMenuCache = {};

export function setJumppadMenuPlayer(playerId, jumppadLocation) {
    const string = JSON.stringify(jumppadLocation)
    jumppadMenuCache[string] = playerId
}

export function getJumppadMenuPlayer(jumppadLocation, dimension) {
    if (dimension.getBlock(jumppadLocation).typeId.startsWith("vf_jp")) {
        const string = JSON.stringify(jumppadLocation)
        return jumppadMenuCache[string]
    } else {
        deleteJumppadMenuPlayer(jumppadLocation)
        return undefined
    }
}

export function deleteJumppadMenuPlayer(jumppadLocation, clearUi = false) {
    const string = JSON.stringify(jumppadLocation)
    if (clearUi === true) {
        if (jumppadMenuCache[string] !== undefined) {
            const player = world.getEntity(jumppadMenuCache[string])
            uiManager.closeAllForms(player)
        }
    }
    delete jumppadMenuCache[string]
}

export function deleteJumppadMenuForPlayer(playerId) {
    // console.error(JSON.stringify(jumppadMenuCache))
    for (const [key, value] of Object.entries(jumppadMenuCache)) {
        // console.error(value)
        if (value === playerId) {
            // console.error("test")
            deleteJumppadMenuPlayer(JSON.parse(key), true)
            return
        }
    }
}

const fallDamageDisableCache = {}

export function addEntityToFallDamageDisable(entityId) {
    fallDamageDisableCache[entityId] = system.currentTick
}

export function removePlayerFromCache() {
    for (const [playerId, currentTick] of Object.entries(fallDamageDisableCache)) {
        // const equippableComponent = world.getEntity(playerId).getComponent('equippable');
        // if (equippableComponent.getEquipment(EquipmentSlot.Chest) !== undefined &&
        //     equippableComponent.getEquipment(EquipmentSlot.Chest).typeId === "minecraft:elytra") {
        //     delete fallDamageDisableCache[playerId]
        //     return
        // }
        if (system.currentTick - currentTick > 5) {
            const player = world.getEntity(playerId)
            // let status = false
            // for (let i = 0; i <= 10; i++) {
            //     try {
            //         if (!player.dimension.getBlock(player.location).below(i).isAir) {
            //             status = true
            //         }
            //     } catch (e) {}
            // }
            // console.error(player.isOnGround)
            if (player.isOnGround === false && player.isInWater === false) {
                player.addEffect("resistance", 30, {amplifier: 20, showParticles: false})
            } else {
                delete fallDamageDisableCache[playerId]
            }
        }
    }
}