import {addEntityToFallDamageDisable, getJumpPadSettings} from "../dataOperations";
import {degreesToVector} from "../utils";
import {MolangVariableMap, system} from "@minecraft/server";
import {
    firstTypeParticlesValues,
    firstTypeSoundsValues,
    secondTypeParticlesValues,
    secondTypeSoundsValues,
    thirdTypeParticlesValues,
    trailAndParticleColorsValues
} from "../config/mainConfig";
import {calculateOffset, getLocationForParticles, invertVector} from "../trailSpawner";

const entityCounter = new Map();

export function jumpPadPush(event) {
    const jumpPadSettings = getJumpPadSettings(event.block.location);

    let directionX = degreesToVector(jumpPadSettings.d).x
    let directionZ = degreesToVector(jumpPadSettings.d).z
    if (jumpPadSettings.pVD === 1) {
        const playerRotation = event.entity.getRotation().y
        const playerDegrees = playerRotationConvertToDegrees(playerRotation)
        directionX = degreesToVector(playerDegrees).x
        directionZ = degreesToVector(playerDegrees).z
    }
    try {
        if (jumpPadSettings.fDS === 0) {
            addEntityToFallDamageDisable(event.entity.id)
        }

        event.entity.applyKnockback(directionX, directionZ, jumpPadSettings.hS, jumpPadSettings.vS)
        const firstTypeSoundIndex = jumpPadSettings.fTS
        if (firstTypeSoundIndex !== -1 && firstTypeSoundIndex !== undefined) {
            // console.error(JSON.stringify(firstTypeSoundsValues[jumpPadSettings.fTS]))
            event.entity.dimension.playSound(firstTypeSoundsValues[jumpPadSettings.fTS].id, event.block.above().center(),
                {
                    pitch: firstTypeSoundsValues[jumpPadSettings.fTS].pitch,
                    volume: firstTypeSoundsValues[jumpPadSettings.fTS].volume
                }
            )
        }

        if (jumpPadSettings.fTP !== -1 && jumpPadSettings.fTP !== undefined) {
            event.dimension.spawnParticle(firstTypeParticlesValues[jumpPadSettings.fTP].id, event.block.above().center())
        }
        if (jumpPadSettings.sTP !== -1 && jumpPadSettings.sTP !== undefined) {
            const color = {
                red: trailAndParticleColorsValues[jumpPadSettings.sTPC].r / 255,
                green: trailAndParticleColorsValues[jumpPadSettings.sTPC].g / 255,
                blue: trailAndParticleColorsValues[jumpPadSettings.sTPC].b / 255
            }
            let frequency = 2
            if (secondTypeParticlesValues[jumpPadSettings.sTP].frequency !== undefined) {
                frequency = secondTypeParticlesValues[jumpPadSettings.sTP].frequency
            }
            repeatParticle(event.entity, secondTypeParticlesValues[jumpPadSettings.sTP].id, color, frequency)
        }

        const thirdTypeParticleIndex = jumpPadSettings.tTP
        const secondTypeSoundIndex = jumpPadSettings.sTS

        let thirdTypeParticleId = undefined
        let secondTypeSoundId = undefined
        let secondTypeSoundPitch = undefined
        let secondTypeSoundVolume = undefined

        if (thirdTypeParticleIndex !== -1 && thirdTypeParticleIndex !== undefined) {
            thirdTypeParticleId = thirdTypeParticlesValues[jumpPadSettings.tTP].id
        }
        if (secondTypeSoundIndex !== -1 && secondTypeSoundIndex !== undefined) {
            secondTypeSoundId = secondTypeSoundsValues[jumpPadSettings.sTS].id
            secondTypeSoundPitch = secondTypeSoundsValues[jumpPadSettings.sTS].pitch
            secondTypeSoundVolume = secondTypeSoundsValues[jumpPadSettings.sTS].volume
        }

        checkEntityOnGround(event.entity, thirdTypeParticleId, secondTypeSoundId,
            secondTypeSoundPitch, secondTypeSoundVolume)
    } catch (e) {
        const argumentObject = {
            directionX: degreesToVector(jumpPadSettings.dFD).x,
            directionZ: degreesToVector(jumpPadSettings.dFD).z,
            maxImpulseCount: jumpPadSettings.mIC,
            tickInterval: jumpPadSettings.tI,
            verticalStrength: jumpPadSettings.vSFD,
            entity: event.entity
        }
        argumentObject.entity.tryTeleport(event.block.center(), {
            keepVelocity: false
        })
        argumentObject.entity.clearVelocity()
        repeatImpulse(argumentObject);

        // console.error(e)
    }
}

function repeatParticle(entity, particle, color, frequency) {
    const molang = new MolangVariableMap()
    molang.setColorRGB("variable.color", color)
    let variableDirection = invertVector(entity.getViewDirection())
    molang.setVector3("variable.direction", variableDirection)
    const newLocationForParticles = getLocationForParticles(entity.location, entity.getViewDirection())
    const offset = calculateOffset(entity.location, newLocationForParticles)
    molang.setVector3("variable.offset", offset)
    system.runTimeout(() => {
        try {
            entity.dimension.spawnParticle(particle, entity.location, molang)
            if (entity.dimension.getBlock(entity.location).below().isAir) {
                repeatParticle(entity, particle, color, frequency)
            }
        } catch (e) {
            repeatParticle(entity, particle, color, frequency)
        }
    }, frequency);
}

function checkEntityOnGround(entity, particle, sound, pitch, volume) {
    system.runTimeout(() => {
        try {
            if (!entity.dimension.getBlock(entity.location).below().isAir) {
                if (particle !== undefined) {
                    entity.dimension.spawnParticle(particle, entity.location)
                }
                if (sound !== undefined) {
                    entity.dimension.playSound(sound, entity.location,
                        {
                            pitch: pitch,
                            volume: volume
                        })
                }
            } else {
                checkEntityOnGround(entity, particle, sound, pitch, volume)
            }
        } catch (e) {
            checkEntityOnGround(entity, particle, sound, pitch, volume)
        }
    }, 5);
}

function repeatImpulse(argumentObject) {
    try {
        const {
            directionX, directionZ, maxImpulseCount,
            tickInterval, verticalStrength, entity
        } = argumentObject
        if (entityCounter.has(entity.id) === false) {
            entityCounter.set(entity.id, 0);
        }

        const count = entityCounter.get(entity.id);
        if (count < maxImpulseCount) {
            // console.error(count)
            entity.clearVelocity()
            // console.error("verticalStrengthForDrop = " + verticalStrengthForDrop);
            entity.applyImpulse({
                x: directionX,
                y: verticalStrength,
                z: directionZ,
            });

            entityCounter.set(entity.id, count + 1)

            system.runTimeout(() => {
                repeatImpulse(argumentObject);
            }, tickInterval);
        } else {
            // console.error("delete")
            entityCounter.delete(entity.id);
        }
    } catch (e) {}
}

function playerRotationConvertToDegrees(playerRotation) {
    if (playerRotation < 0) {
        return playerRotation + 360
    }
    return playerRotation
}