import {getDefaultSettings, getJumpPadSettings} from "./dataOperations";
import {ItemStack, system} from "@minecraft/server";
import {
    defaultDegrees,
    defaultDegreesForDrop,
    defaultFirstTypeParticleId,
    defaultFirstTypeSoundId,
    defaultHS,
    defaultMaxImpulseCount,
    defaultPushViewDirection,
    defaultSecondTypeParticleColorId,
    defaultSecondTypeParticleId,
    defaultSecondTypeSoundId,
    defaultThirdTypeParticleId,
    defaultTickInterval,
    defaultVS,
    defaultVSForDrop
} from "./config/mainConfig";

const forPlayerNaming = {
    d: 'Degrees',

    pVD: 'Push View Direction',

    hS: 'Horizontal Strength',
    vS: 'Vertical Strength',
    mIC: 'Max Impulse Count',
    tI: 'Tick Interval',
    vSFD: 'Vertical Strength For Drop',
    dFD: 'Degrees For Drop',

    fTP: 'First Type Particle Id',
    sTP: 'Second Type Particle Id',
    tTP: 'Third Type Particle Id',

    fTS: 'First Type Sound Id',
    sTS: 'Second Type Sound Id',

    sTPC: 'Second Type Particle Color'
}

export function setJumpPadSettingsInLoreAndDrop(event) {
    const itemStackTypeId = event.block.typeId + "_item"
    const itemStack = new ItemStack(itemStackTypeId, 1);
    const settings = getJumpPadSettings(event.block.location)
    system.runTimeout(() => {
        const newLore = []

        for (const [key, value] of Object.entries(settings)) {
            switch (key) {
                case "d":
                    if (value === defaultDegrees) {
                        continue
                    }
                    break
                case "pVD":
                    if (value === defaultPushViewDirection) {
                        continue
                    }
                    break
                case "hS":
                    if (value === defaultHS) {
                        continue
                    }
                    break
                case "vS":
                    if (value === defaultVS) {
                        continue
                    }
                    break
                case "mIC":
                    if (value === defaultMaxImpulseCount) {
                        continue
                    }
                    break
                case "tI":
                    if (value === defaultTickInterval) {
                        continue
                    }
                    break
                case "vSFD":
                    if (value === defaultVSForDrop) {
                        continue
                    }
                    break
                case "dFD":
                    if (value === defaultDegreesForDrop) {
                        continue
                    }
                    break
                case "fTP":
                    if (value === defaultFirstTypeParticleId) {
                        continue
                    }
                    break
                case "sTP":
                    if (value === defaultSecondTypeParticleId) {
                        continue
                    }
                    break
                case "tTP":
                    if (value === defaultThirdTypeParticleId) {
                        continue
                    }
                    break
                case "fTS":
                    if (value === defaultFirstTypeSoundId) {
                        continue
                    }
                    break
                case "sTS":
                    if (value === defaultSecondTypeSoundId) {
                        continue
                    }
                    break
                case "sTPC":
                    if (value === defaultSecondTypeParticleColorId) {
                        continue
                    }
                    break
            }
            newLore.push(`§8${forPlayerNaming[key]}: ${value}`)
        }

        itemStack.setLore(newLore);

        event.player.dimension.spawnItem(itemStack, event.block.location);
    }, 1);
}

export function parseSettingsFromLore(lore) {
    const defaultSettings = getDefaultSettings()

    const parsedSettings = { ...defaultSettings };

    const reverseNaming = Object.fromEntries(
        Object.entries(forPlayerNaming).map(([key, displayName]) => [displayName, key])
    );

    lore.forEach(line => {
        const match = line.match(/^§8(.*?):\s*(.+)$/);
        if (match) {
            const [, displayName, value] = match;
            const key = reverseNaming[displayName];
            if (key) {
                parsedSettings[key] = isNaN(value) ? value : Number(value); // Парсим число или строку
            }
        }
    });

    return parsedSettings;
}