import {BlockPermutation, EntityComponentTypes, EquipmentSlot, ItemStack, system, world} from "@minecraft/server";
import {
    deleteJumppadMenuForPlayer,
    deleteJumppadMenuPlayer,
    deleteJumpPadSettings,
    getJumppadMenuPlayer,
    getJumpPadSettings,
    getTrailSettings,
    removePlayerFromCache,
    setJumpPadSettings,
    trailPropertyId
} from "./dataOperations";
import {openMainJumpPadMenu} from "./menu/openMainMenu";
import {jumpPadPush} from "./jumpPad/jumpPadPush";
import {openTrailsMenu} from "./menu/openTrailsMenu";
import {trailSpawner} from "./trailSpawner";
import {parseSettingsFromLore, setJumpPadSettingsInLoreAndDrop} from "./jumpPadSettingsLore";
import {trailParticlesValues} from "./config/mainConfig";
import {openColoredTrailsMenu} from "./menu/openColoredMenu";

//----------------------------------------------------------------------------------------------------------------------
//TutorialBook
world.afterEvents.playerSpawn.subscribe(checkTutorialBook);

function checkTutorialBook(event) {
    if (event.player.getDynamicProperty("vf_jp:tutorialbook") !== undefined
        && event.player.getDynamicProperty("vf_jp:tutorialbook") === "true") {
        return;
    }
    event.player.dimension.spawnItem(new ItemStack("vf_jp:tutorial_book0", 1),
        event.player.location);
    event.player.setDynamicProperty("vf_jp:tutorialbook", "true");
}

function playPageTurnSound(player) {
    player.playSound("item.book.page_turn");
}


world.afterEvents.itemUse.subscribe(pageTurnBookMinus);

function pageTurnBookMinus(event) {
    // console.error(JSON.stringify(event.source.location));
    if (!event.source.isSneaking) {
        return;
    }

    const playerMainHandBoolean = event.source.getComponent(EntityComponentTypes.Equippable)
        .getEquipment(EquipmentSlot.Mainhand).typeId.startsWith("vf_jp:tutorial_book");
    const playerMainHandTypeId = event.source.getComponent(EntityComponentTypes.Equippable)
        .getEquipment(EquipmentSlot.Mainhand).typeId;

    if (playerMainHandTypeId === undefined || !playerMainHandBoolean) {
        return;
    }


    playPageTurnSound(event.source);

    const currentTutorialBookPage = playerMainHandTypeId.replace(/^vf_jp:tutorial_book/, '');
    let newTutorialBookPage = parseInt(currentTutorialBookPage) - 1;
    if (playerMainHandTypeId === "vf_jp:tutorial_book0") {
        newTutorialBookPage = 3
    }


    event.source.getComponent(EntityComponentTypes.Equippable).setEquipment(EquipmentSlot.Mainhand,
        new ItemStack(`vf_jp:tutorial_book${newTutorialBookPage}`));
}

world.afterEvents.itemUse.subscribe(pageTurnBookPlus);

function pageTurnBookPlus(event) {
    if (event.source.isSneaking) {
        return;
    }

    const playerMainHandBoolean = event.source.getComponent(EntityComponentTypes.Equippable)
        .getEquipment(EquipmentSlot.Mainhand).typeId.startsWith("vf_jp:tutorial_book");
    const playerMainHandTypeId = event.source.getComponent(EntityComponentTypes.Equippable)
        .getEquipment(EquipmentSlot.Mainhand).typeId;

    if (playerMainHandTypeId === undefined || !playerMainHandBoolean) {
        return;
    }

    playPageTurnSound(event.source);

    const currentTutorialBookPage = playerMainHandTypeId.replace(/^vf_jp:tutorial_book/, '');
    let newTutorialBookPage = parseInt(currentTutorialBookPage) + 1;
    if (playerMainHandTypeId === "vf_jp:tutorial_book3") {
        newTutorialBookPage = 0
    }

    event.source.getComponent(EntityComponentTypes.Equippable).setEquipment(EquipmentSlot.Mainhand,
        new ItemStack(`vf_jp:tutorial_book${newTutorialBookPage}`));
}

//----------------------------------------------------------------------------------------------------------------------
//Jumppad events
world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('vf_jp:jumppad_onStepOn', {
        onStepOn: event => {
            jumpPadPush(event)
            const typeId = event.block.typeId
            const blockPermutation = BlockPermutation.resolve(typeId)
            if (!typeId.endsWith("_down") && typeId.startsWith("vf_jp:jumppad_custom_")) {
                const typeId = event.block.typeId
                const newTypeIdForPerm = typeId + "_down"
                const newBlockPermutation = BlockPermutation.resolve(newTypeIdForPerm)
                event.dimension.setBlockPermutation(event.block.location, newBlockPermutation)
                system.runTimeout(() => {
                        try {
                            if (event.dimension.getBlock(event.block.location).typeId === newTypeIdForPerm) {
                                event.dimension.setBlockPermutation(event.block.location, blockPermutation)
                            }
                        } catch (e) {
                        }
                    }
                    , 20);
            } else if (typeId.startsWith("vf_jp:jumppad_custom_")) {
                const newTypeIdForPerm = typeId.slice(0, -"_down".length)
                const newBlockPermutation = BlockPermutation.resolve(newTypeIdForPerm)
                system.runTimeout(() => {
                        try {
                            if (event.dimension.getBlock(event.block.location).typeId === typeId) {
                                event.dimension.setBlockPermutation(event.block.location, newBlockPermutation)
                            }
                        } catch (e) {
                        }
                    }
                    , 10);
            }
        },
    });
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('vf_jp:jumppad_onPlayerInteract', {
        onPlayerInteract: event => {
            const argumentObject = {
                player: event.player,
                blockLocation: event.block.location,
            }
            if (getJumppadMenuPlayer(argumentObject.blockLocation, event.player.dimension) === undefined) {
                openMainJumpPadMenu(argumentObject)
                // console.error("test1")
            } else {
                const playerId = getJumppadMenuPlayer(argumentObject.blockLocation, event.player.dimension)
                const player = world.getEntity(playerId)
                if (player === undefined) {
                    deleteJumppadMenuPlayer(argumentObject.blockLocation)
                    openMainJumpPadMenu(argumentObject)
                    // console.error("test2")
                } else {
                    argumentObject.player.sendMessage("This jumppad is already being configured by another player.")
                    // console.error("test3")
                }
            }
        },
    });
});
//----------------------------------------------------------------------------------------------------------------------
//delete settings
world.afterEvents.playerBreakBlock.subscribe(initEvent => {
    deleteJumppadMenuPlayer(initEvent.block.location, true)
    deleteJumpPadSettings(initEvent.block.location)
});

//----------------------------------------------------------------------------------------------------------------------
//trails
world.afterEvents.itemUse.subscribe(initEvent => {
        // console.error(initEvent.itemStack.typeId)
        if (initEvent.source.typeId !== "minecraft:player") {
            return;
        }

        if (initEvent.itemStack.typeId !== "vf_jp:jumppad_trails_stick") {
            return;
        }

        if (!initEvent.source.isSneaking) {
            openTrailsMenu(initEvent.source)
        } else {
            const playerHasTrail = JSON.parse(initEvent.source.getDynamicProperty(trailPropertyId)).trailId !== undefined
            if (playerHasTrail) {
                const settings = getTrailSettings(initEvent.source)
                if (trailParticlesValues[settings.trailId].colored !== undefined) {
                    openColoredTrailsMenu(initEvent.source, settings.trailId)
                }
            }
        }
    }
)

system.runInterval(trailSpawner, 1);
//----------------------------------------------------------------------------------------------------------------------
//jumppad settings
world.beforeEvents.playerBreakBlock.subscribe(initEvent => {
    if (initEvent.block.typeId.startsWith("vf_jp:") && initEvent.player.getGameMode() !== "creative") {
        setJumpPadSettingsInLoreAndDrop(initEvent)
    }
});

const playerItemUse = new Map()

world.afterEvents.itemStartUseOn.subscribe(initEvent => {
    try {
        if (initEvent.itemStack.typeId.startsWith("vf_jp:jumppad_") &&
            playerItemUse.get(initEvent.source.id).currentTick === system.currentTick) {
            const lore = initEvent.itemStack.getLore()
            setJumpPadSettings(playerItemUse.get(initEvent.source.id).location, parseSettingsFromLore(lore))
        } else {
            playerItemUse.delete(initEvent.source.id)
        }
    } catch (e) {
    }
})

world.afterEvents.playerPlaceBlock.subscribe(initEvent => {
    try {
        if (initEvent.block.typeId.startsWith("vf_jp:jumppad_") &&
            initEvent.player.typeId === "minecraft:player") {
            const data = {
                location: initEvent.block.location,
                currentTick: system.currentTick
            }
            playerItemUse.set(initEvent.player.id, data)
        } else {
            playerItemUse.delete(initEvent.player.id)
        }
    } catch (e) {
    }
});

//----------------------------------------------------------------------------------------------------------------------
//piston
world.afterEvents.pistonActivate.subscribe(initEvent => {
    const isExpanding = initEvent.isExpanding
    const dimension = initEvent.dimension
    const blockLocations = initEvent.piston.getAttachedBlocksLocations()

    for (const oldJumpPadLocation of blockLocations) {

        const newBlockLocation = moveLocation(initEvent.block, oldJumpPadLocation, isExpanding)
        deleteJumppadMenuPlayer(oldJumpPadLocation, true)
        deleteJumppadMenuPlayer(newBlockLocation, true)
        // dimension.spawnParticle("minecraft:heart_particle", dimension.getBlock(newBlockLocation).above().center())
        system.runTimeout(() => {
                const blockTypeId = dimension.getBlock(newBlockLocation).typeId

                if (blockTypeId.startsWith("vf_jp:jumppad_")) {
                    // console.error(blockTypeId)
                    const oldLocationSettings = getJumpPadSettings(oldJumpPadLocation)
                    setJumpPadSettings(newBlockLocation, oldLocationSettings)
                } else if (!dimension.getBlock(oldJumpPadLocation).typeId.startsWith("vf_jp:jumppad_")) {
                    deleteJumpPadSettings(oldJumpPadLocation)
                }
            }
            , 2);
    }

})

function moveLocation(pistonBlock, jumppadLoc, isExpanding) {
    const direction = pistonBlock.permutation.getState('facing_direction')
    const result = {...jumppadLoc};
    // console.error(direction)
    switch (direction) {
        case 4: {
            let delta = isExpanding ? 1 : -1;
            result["x"] = result["x"] + delta;
            result["y"] = result["y"];
            result["z"] = result["z"];
            break;
        }
        case 2: {
            let delta = isExpanding ? 1 : -1;
            result["x"] = result["x"];
            result["y"] = result["y"];
            result["z"] = result["z"] + delta;
            break;
        }
        case 5: {
            let delta = isExpanding ? -1 : 1;
            result["x"] = result["x"] + delta;
            result["y"] = result["y"];
            result["z"] = result["z"];
            break;
        }
        case 3: {
            let delta = isExpanding ? -1 : 1;
            result["x"] = result["x"];
            result["y"] = result["y"];
            result["z"] = result["z"] + delta;
            break;
        }
        case 1: {
            let delta = isExpanding ? 1 : -1;
            result["x"] = result["x"];
            result["y"] = result["y"] + delta;
            result["z"] = result["z"];
            break;
        }
        case 0: {
            let delta = isExpanding ? -1 : 1;
            result["x"] = result["x"];
            result["y"] = result["y"] + delta;
            result["z"] = result["z"];
            break;
        }
    }
    return result;
}

//----------------------------------------------------------------------------------------------------------------------
//clear on explode
world.afterEvents.blockExplode.subscribe(initEvent => {
    const location = initEvent.block.location
    deleteJumppadMenuPlayer(location, true)
})

//----------------------------------------------------------------------------------------------------------------------
//player die
world.afterEvents.entityDie.subscribe(initEvent => {
    if (initEvent.deadEntity.typeId === "minecraft:player") {
        // console.error("test")
        deleteJumppadMenuForPlayer(initEvent.deadEntity.id)
    }
})

//----------------------------------------------------------------------------------------------------------------------
//entity hurt
system.runInterval(removePlayerFromCache, 1);