import {ModalFormData} from "@minecraft/server-ui";
import {
    deleteJumppadMenuPlayer,
    getJumpPadSettings,
    setJumppadMenuPlayer,
    setJumpPadSettings
} from "../../../dataOperations";

export function openFineTunedMenuForEntities(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const modalForm = new ModalFormData().title("Templated Settings");
    const jumpPadSettings = getJumpPadSettings(argumentObject.blockLocation);

    const {
        maxImpulseCount, tickInterval, verticalStrengthForDrop, degreesForDrop
    } = {
        maxImpulseCount: jumpPadSettings.mIC,
        tickInterval: jumpPadSettings.tI,
        verticalStrengthForDrop: jumpPadSettings.vSFD,
        degreesForDrop: jumpPadSettings.dFD
    };

    modalForm.slider("Here, you need to set the exact direction in which the jump pad will push." +
        " Try different values to understand how it works.\n\nValue" , 0, 359.5, 0.5, degreesForDrop);

    modalForm.textField("Set vertical push strength for drops and items:", "Vertical Strength For Drop",
        verticalStrengthForDrop.toString());

    modalForm.textField("Set the max number of pushes for an item:" + "§8 \n(the item will be pushed with the same force and direction)", "Number",
        maxImpulseCount.toString());

    modalForm.textField("Set the delay between repeated pushes:" + "§8 \n(the number must be an integer)", "Pushes",
        tickInterval.toString());

    modalForm
        .show(argumentObject.player)
        .then((formData) => {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            if (formData.cancelationReason === "UserClosed") {
                argumentObject.player.sendMessage("Menu closed without applying changes.")
                return
            }
            if (isNaN(parseFloat(formData.formValues[2].toString().replaceAll(",", "."))) ||
                isNaN(parseInt(formData.formValues[3].toString().replaceAll(",", "."))) ||
                isNaN(parseFloat(formData.formValues[1].toString().replaceAll(",", "."))) ||
                isNaN(parseFloat(formData.formValues[0].toString()))) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return
            }

            // const settings = {
            //     d: degrees,
            //     hS: horizontalStrength,
            //     vS: verticalStrength,
            //     mIC: parseFloat(formData.formValues[2].toString()),
            //     tI: parseFloat(formData.formValues[3].toString()),
            //     vSFD: parseFloat(formData.formValues[1].toString()),
            //     dFD: parseFloat(formData.formValues[0].toString())
            // }
            const settings = Object.assign({}, jumpPadSettings);
            settings.mIC = parseFloat(formData.formValues[2].toString().replaceAll(",", "."))
            settings.tI = parseInt(formData.formValues[3].toString().replaceAll(",", "."))
            settings.vSFD = parseFloat(formData.formValues[1].toString().replaceAll(",", "."))
            settings.dFD = parseFloat(formData.formValues[0].toString())

            setJumpPadSettings(argumentObject.blockLocation, settings)
        })
        .catch((error) => {
            argumentObject.player.sendMessage("An error occurred while processing the entered data.")
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return -1;
        });
}