import {ModalFormData} from "@minecraft/server-ui";
import {
    templatedDegOpt,
    templatedMCForDropOpt,
    templatedTIForDropOpt,
    templatedVSForDropOpt
} from "../../../config/mainConfig";
import {
    deleteJumppadMenuPlayer,
    getJumpPadSettings,
    setJumppadMenuPlayer,
    setJumpPadSettings
} from "../../../dataOperations";
import {findClosestNumber} from "../../../utils";

export function openTemplatedMenuForEntities(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const modalForm = new ModalFormData().title("Templated Settings");
    const jumpPadSettings = getJumpPadSettings(argumentObject.blockLocation);

    const {
        maxImpulseCount, tickInterval, verticalStrengthForDrop, degreesForDrop
    } = {
        maxImpulseCount: jumpPadSettings.mIC,
        tickInterval: jumpPadSettings.tI,
        verticalStrengthForDrop: jumpPadSettings.vSFD,
        degreesForDrop: jumpPadSettings.dFD
    };

    modalForm.dropdown("Here, you need to set the exact direction in which the jump pad will push.. " +
        "Try different values to understand how it works.", templatedDegOpt,
        templatedDegOpt.indexOf(findClosestNumber(degreesForDrop, templatedDegOpt).toString()));

    modalForm.dropdown("Set vertical push strength for drops and items:", templatedVSForDropOpt,
        templatedVSForDropOpt.indexOf(verticalStrengthForDrop.toString()));
    // console.error(verticalStrengthForDrop)

    modalForm.dropdown("Set the max number of pushes for an item:" + "§8 \n(the item will be pushed with the same force and direction)", templatedMCForDropOpt,
        templatedMCForDropOpt.indexOf(maxImpulseCount.toString()));

    modalForm.dropdown("Set the delay between repeated pushes:", templatedTIForDropOpt,
        templatedTIForDropOpt.indexOf(tickInterval.toString()));

    modalForm
        .show(argumentObject.player)
        .then((formData) => {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            if (formData.cancelationReason === "UserClosed") {
                argumentObject.player.sendMessage("Menu closed without applying changes.")
                return
            }
            // const settings = {
            //     d: degrees,
            //     hS: horizontalStrength,
            //     vS: verticalStrength,
            //     mIC: parseFloat(templatedMCForDropOpt[formData.formValues[2]]),
            //     tI: parseFloat(templatedTIForDropOpt[formData.formValues[3]]),
            //     vSFD: parseFloat(templatedVSForDropOpt[formData.formValues[1]]),
            //     dFD: parseFloat(templatedDegOpt[formData.formValues[0]])
            // }
            const settings = Object.assign({}, jumpPadSettings);
            settings.mIC = parseFloat(templatedMCForDropOpt[formData.formValues[2]])
            settings.tI = parseFloat(templatedTIForDropOpt[formData.formValues[3]])
            settings.vSFD = parseFloat(templatedVSForDropOpt[formData.formValues[1]])
            settings.dFD = parseFloat(templatedDegOpt[formData.formValues[0]])
            // console.error(parseFloat(templatedVSForDropOpt[formData.formValues[1]]))
            // console.error(JSON.stringify(settings, null, 2))

            setJumpPadSettings(argumentObject.blockLocation, settings)
        })
        .catch((error) => {
            argumentObject.player.sendMessage("An error occurred while processing the entered data.")
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return -1;
        });
}