import {ModalFormData} from "@minecraft/server-ui";
import {
    deleteJumppadMenuPlayer,
    getJumpPadSettings,
    setJumppadMenuPlayer,
    setJumpPadSettings
} from "../../../dataOperations";

export function openFineTunedMenuForPlayer(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const modalForm = new ModalFormData().title("Fine Tuned Settings");

    const jumpPadSettings = getJumpPadSettings(argumentObject.blockLocation);

    const {
        degrees,pushViewDirection, horizontalStrength, verticalStrength, fallDamageStatus
    } = {
        degrees: jumpPadSettings.d,
        pushViewDirection: jumpPadSettings.pVD,
        horizontalStrength: jumpPadSettings.hS,
        verticalStrength: jumpPadSettings.vS,
        fallDamageStatus: jumpPadSettings.fDS
    }

    modalForm.slider("Here, you need to set the exact direction in which the jump pad will push." +
    " Try different values to understand how it works.\n\nValue" , 0, 359.5, 0.5, degrees);

    let toggleValue = false
    if (pushViewDirection === 1) {
        toggleValue = true
    }

    modalForm.toggle("Push player and entities based on their view direction.\n" + "§4Enabling this will override the set direction!", toggleValue);

    modalForm.textField("Set horizontal push strength \n§4(no more than 1000)§f:", "Horizontal Strength",
        horizontalStrength.toString());

    modalForm.textField("Set vertical push strength \n§4(no more than 500)§f:", "Vertical Strength",
        verticalStrength.toString());

    let fallValue = false
    if (fallDamageStatus === 1) {
        fallValue = true
    }

    modalForm.toggle("Fall damage?", fallValue);

    modalForm
        .show(argumentObject.player)
        .then((formData) => {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            if (formData.cancelationReason === "UserClosed") {
                argumentObject.player.sendMessage("Menu closed without applying changes.")
                return
            }
            if (isNaN(parseFloat(formData.formValues[2].toString().replaceAll(",", "."))) ||
                isNaN(parseFloat(formData.formValues[3].toString().replaceAll(",", ".")))) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return
            }
            if (parseFloat(formData.formValues[2].toString().replaceAll(",", ".")) > 1000 ||
                parseFloat(formData.formValues[3].toString().replaceAll(",", ".")) > 500) {
                argumentObject.player.sendMessage("One of the arguments exceeds the allowed number. Menu closed without applying changes.")
                return
            }

            let newDegrees = formData.formValues[0]
            let newPushViewDirection = 0
            if (formData.formValues[1] === true) {
                newPushViewDirection = 1
            }

            const settings = Object.assign({}, jumpPadSettings);
            settings.d = newDegrees
            settings.pVD = newPushViewDirection
            settings.hS = parseFloat(formData.formValues[2].toString().replaceAll(",", "."))
            settings.vS = parseFloat(formData.formValues[3].toString().replaceAll(",", "."))

            let newFallDamageStatus = 0
            if (formData.formValues[4] === true) {
                newFallDamageStatus = 1
            }
            settings.fDS = newFallDamageStatus

            setJumpPadSettings(argumentObject.blockLocation, settings)
        })
        .catch((error) => {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            argumentObject.player.sendMessage("An error occurred while processing the entered data.")
            return -1;
        });
}