import {ModalFormData} from "@minecraft/server-ui";
import {templatedDegOpt, templatedHSOpt, templatedVSOpt} from "../../../config/mainConfig";
import {
    deleteJumppadMenuPlayer,
    getJumpPadSettings,
    setJumppadMenuPlayer,
    setJumpPadSettings
} from "../../../dataOperations";
import {findClosestNumber} from "../../../utils";

export function openTemplatedMenuForPlayer(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const modalForm = new ModalFormData().title("Templated Settings");
    const jumpPadSettings = getJumpPadSettings(argumentObject.blockLocation);

    const {
        degrees,pushViewDirection, horizontalStrength, verticalStrength, fallDamageStatus
    } = {
        degrees: jumpPadSettings.d,
        pushViewDirection: jumpPadSettings.pVD,
        horizontalStrength: jumpPadSettings.hS,
        verticalStrength: jumpPadSettings.vS,
        fallDamageStatus: jumpPadSettings.fDS
    }

    modalForm.dropdown("Here, you need to set the exact direction in which the jump pad will push." + " Try different values to understand how it works.", templatedDegOpt,
        templatedDegOpt.indexOf(findClosestNumber(degrees, templatedDegOpt).toString()));

    let toggleValue = false
    if (pushViewDirection === 1) {
        toggleValue = true
    }

    modalForm.toggle("Push player and entities based on their view direction? " +
        "§4Enabling this will override the set direction!", toggleValue);

    modalForm.dropdown("Set horizontal push strength:", templatedHSOpt,
        templatedHSOpt.indexOf(findClosestNumber(horizontalStrength, templatedHSOpt).toString()));

    modalForm.dropdown("Set vertical push strength:", templatedVSOpt,
        templatedVSOpt.indexOf(findClosestNumber(verticalStrength, templatedVSOpt).toString()));

    let fallValue = false
    if (fallDamageStatus === 1) {
        fallValue = true
    }

    modalForm.toggle("Fall damage?", fallValue);

    modalForm
        .show(argumentObject.player)
        .then((formData) => {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            if (formData.cancelationReason === "UserClosed") {
                argumentObject.player.sendMessage("Menu closed without applying changes.")
                return
            }
            let newDegrees = parseFloat(templatedDegOpt[formData.formValues[0]])
            let newPushViewDirection
            if (formData.formValues[1] === true) {
                newPushViewDirection = 1
            }
            // const settings = {
            //     d: newDegrees,
            //     hS: parseFloat(templatedHSOpt[formData.formValues[2]]),
            //     vS: parseFloat(templatedVSOpt[formData.formValues[3]]),
            //     mIC: maxImpulseCount,
            //     tI: tickInterval,
            //     vSFD: verticalStrengthForDrop,
            //     dFD: degreesForDrop
            // }
            const settings = Object.assign({}, jumpPadSettings);
            settings.d = newDegrees
            settings.pVD = newPushViewDirection
            settings.hS = parseFloat(templatedHSOpt[formData.formValues[2]])
            settings.vS = parseFloat(templatedVSOpt[formData.formValues[3]])

            let newFallDamageStatus = 0
            if (formData.formValues[4] === true) {
                newFallDamageStatus = 1
            }
            settings.fDS = newFallDamageStatus

            setJumpPadSettings(argumentObject.blockLocation, settings)
            // console.error(JSON.stringify(settings, null, 2))
        })
        .catch((error) => {
            argumentObject.player.sendMessage("An error occurred while processing the entered data.")
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return -1;
        });
}