import {ActionFormData} from "@minecraft/server-ui";
import {trailAndParticleColorsKeys, trailAndParticleColorsValues, trailParticlesKeys} from "../config/mainConfig";
import {
    deleteJumppadMenuPlayer,
    getJumpPadSettings,
    setJumppadMenuPlayer,
    setJumpPadSettings,
    setTrailSettings
} from "../dataOperations";

export function openColoredTrailsMenu(player, particleId) {
    const form = new ActionFormData()
        .title("Select color for \"" + trailParticlesKeys[particleId] + "\"")

    for (const colorName of trailAndParticleColorsKeys) {
        const index = trailAndParticleColorsKeys.indexOf(colorName);
        form.button(colorName, trailAndParticleColorsValues[index].iconPath)
    }

    form.show(player).then(response => {
        if (!response || response.selection === undefined) return;
        try {
            setTrailSettings(player, particleId, response.selection)
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

export function openColoredParticlesMenu(player, particleId, argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select color for \"" + trailParticlesKeys[particleId] + "\"")

    for (const colorName of trailAndParticleColorsKeys) {
        const index = trailAndParticleColorsKeys.indexOf(colorName);
        form.button(colorName, trailAndParticleColorsValues[index].iconPath)
    }

    form.show(player).then(response => {
        if (!response || response.selection === undefined) {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            newSettings.sTP = particleId
            newSettings.sTPC = response.selection
            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}