import {ActionFormData} from "@minecraft/server-ui";
import {
    firstTypeParticlesKeys,
    firstTypeParticlesValues,
    secondTypeParticlesKeys,
    secondTypeParticlesValues,
    thirdTypeParticlesKeys,
    thirdTypeParticlesValues
} from "../config/mainConfig";
import {deleteJumppadMenuPlayer, getJumpPadSettings, setJumppadMenuPlayer, setJumpPadSettings} from "../dataOperations";
import {openColoredParticlesMenu} from "./openColoredMenu";
import {openPresetsMenu} from "./openPresetsMenu";

export function openParticlesMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select particles type:")
        .button("Jump moment", "textures/vf/jp/ui_icons/icon_jumppad_up")
        .button("Flight trial", "textures/vf/jp/ui_icons/icon_flight_trail")
        .button("Landing", "textures/vf/jp/ui_icons/icon_jumppad_down")
        .button("§8Presets", "textures/vf/jp/ui_icons/icon_preset")

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined)  {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            switch (response.selection) {
                case 0:
                    openFirstTypeParticlesMenu(argumentObject)
                    break
                case 1:
                    openSecondTypeParticlesMenu(argumentObject)
                    break
                case 2:
                    openThirdTypeParticlesMenu(argumentObject)
                    break
                case 3:
                    openPresetsMenu(argumentObject)
                    break
            }
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

function openFirstTypeParticlesMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select jump particle:")

    const isFTP = getJumpPadSettings(argumentObject.blockLocation).fTP !== -1
    if (isFTP) {
        form.button("§cDisable")
    }

    for (const particleName of firstTypeParticlesKeys) {
        const index = firstTypeParticlesKeys.indexOf(particleName);
        form.button(particleName, firstTypeParticlesValues[index].iconPath)
    }

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined)  {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            if (isFTP) {
                if (response.selection === 0) {
                    newSettings.fTP = -1
                } else {
                    newSettings.fTP = response.selection - 1
                }
            } else {
                newSettings.fTP = response.selection
            }

            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

function openSecondTypeParticlesMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select flight particle:")

    //console.error(getJumpPadSettings(argumentObject.blockLocation).sTP)

    const isSTP = getJumpPadSettings(argumentObject.blockLocation).sTP !== -1
    if (isSTP) {
        form.button("§cDisable")
    }

    for (const particleName of secondTypeParticlesKeys) {
        const index = secondTypeParticlesKeys.indexOf(particleName);
        form.button(particleName, secondTypeParticlesValues[index].iconPath)
    }

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined) {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            if (isSTP) {
                if (secondTypeParticlesValues[response.selection - 1]?.colored !== undefined) {
                    openColoredParticlesMenu(argumentObject.player, response.selection - 1)
                    return
                } else {
                    if (response.selection === 0) {
                        newSettings.sTP = -1
                    } else {
                        newSettings.sTP = response.selection - 1
                    }
                }
            } else {
                if (secondTypeParticlesValues[response.selection].colored !== undefined) {
                    openColoredParticlesMenu(argumentObject.player, response.selection, argumentObject)
                    return
                } else {
                    newSettings.sTP = response.selection
                }
            }

            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

function openThirdTypeParticlesMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select landing particle:")

    const isTTP = getJumpPadSettings(argumentObject.blockLocation).tTP !== -1
    if (isTTP) {
        form.button("§cDisable")
    }

    for (const particleName of thirdTypeParticlesKeys) {
        const index = thirdTypeParticlesKeys.indexOf(particleName);
        form.button(particleName, thirdTypeParticlesValues[index].iconPath)
    }

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined) {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            if (isTTP) {
                if (response.selection === 0) {
                    newSettings.tTP = -1
                } else {
                    newSettings.tTP = response.selection - 1
                }
            } else {
                newSettings.tTP = response.selection
            }

            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}