import {ActionFormData} from "@minecraft/server-ui";
import {deleteJumppadMenuPlayer, getJumpPadSettings, setJumppadMenuPlayer, setJumpPadSettings} from "../dataOperations";
import {firstTypeSoundsKeys, secondTypeSoundsKeys} from "../config/mainConfig";

export function openSoundsMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select sounds type:")
        .button("Jump moment", "textures/vf/jp/ui_icons/icon_jumppad_up")
        .button("Landing", "textures/vf/jp/ui_icons/icon_jumppad_down")

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined) {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            switch (response.selection) {
                case 0:
                    openFirstTypeSoundsMenu(argumentObject)
                    break
                case 1:
                    openSecondTypeSoundsMenu(argumentObject)
                    break
            }
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

function openFirstTypeSoundsMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select jump sound:")

    const isFTS = getJumpPadSettings(argumentObject.blockLocation).fTS !== -1
    if (isFTS) {
        form.button("§cDisable")
    }

    for (const soundName of firstTypeSoundsKeys) {
        form.button(soundName)
    }

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined)  {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            if (isFTS) {
                if (response.selection === 0) {
                    newSettings.fTS = -1
                } else {
                    newSettings.fTS = response.selection - 1
                }
            } else {
                newSettings.fTS = response.selection
            }

            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}

function openSecondTypeSoundsMenu(argumentObject) {
    setJumppadMenuPlayer(argumentObject.player.id, argumentObject.blockLocation)
    const form = new ActionFormData()
        .title("Select jump sound:")

    const isSTS = getJumpPadSettings(argumentObject.blockLocation).sTS !== -1
    if (isSTS) {
        form.button("§cDisable")
    }

    for (const soundName of secondTypeSoundsKeys) {
        form.button(soundName)
    }

    form.show(argumentObject.player).then(response => {
        if (!response || response.selection === undefined)  {
            deleteJumppadMenuPlayer(argumentObject.blockLocation)
            return;
        }
        deleteJumppadMenuPlayer(argumentObject.blockLocation)
        try {
            if (!argumentObject.player.dimension.getBlock(argumentObject.blockLocation).typeId.startsWith("vf_jp:jumppad_")) {
                argumentObject.player.sendMessage("An error occurred while processing the entered data.")
                return;
            }
            const newSettings = getJumpPadSettings(argumentObject.blockLocation)
            if (isSTS) {
                if (response.selection === 0) {
                    newSettings.sTS = -1
                } else {
                    newSettings.sTS = response.selection - 1
                }
            } else {
                newSettings.sTS = response.selection
            }

            setJumpPadSettings(argumentObject.blockLocation, newSettings);
        } catch (e) {
            //console.error("Error:", e);
        }
    });
}