import {MolangVariableMap, system, world} from "@minecraft/server";
import {getTrailSettings} from "./dataOperations";
import {trailAndParticleColorsValues, trailParticlesValues} from "./config/mainConfig";

export function trailSpawner() {
    world.getPlayers().forEach(player => {
        if (getTrailSettings(player).trailId === undefined) {
            return;
        }

        let frequency = 2
        if (trailParticlesValues[getTrailSettings(player).trailId].frequency !== undefined) {
            frequency = trailParticlesValues[getTrailSettings(player).trailId].frequency
        }

        if (!(system.currentTick % frequency === 0)) {
            return;
        }

        const particleId = trailParticlesValues[getTrailSettings(player).trailId].id
        const dimenstion = player.dimension
        const viewDirection = player.getViewDirection()
        let color = {
            red: 0,
            green: 0,
            blue: 0
        }

        if (getTrailSettings(player).colorId !== undefined) {
            color = {
                red: trailAndParticleColorsValues[getTrailSettings(player).colorId].r / 255,
                green: trailAndParticleColorsValues[getTrailSettings(player).colorId].g / 255,
                blue: trailAndParticleColorsValues[getTrailSettings(player).colorId].b / 255
            }
        }
        let variableDirection = invertVector(player.getViewDirection())
        if (trailParticlesValues[getTrailSettings(player).trailId].isBackDirection === true) {
            variableDirection = player.getViewDirection()
        }
        const molang = new MolangVariableMap()
        molang.setColorRGB("variable.color", color)

        molang.setVector3("variable.direction", variableDirection)

        const newLocationForParticles = getLocationForParticles(player.location, viewDirection)

        const offset = calculateOffset(player.location, newLocationForParticles)
        molang.setVector3("variable.offset", offset)

        const newLocationForFlameThrower = getFlameThrowerLocationForParticles(player.location, viewDirection)
        const flameThrowerOffset = calculateOffset(player.location, newLocationForFlameThrower)
        molang.setVector3("variable.offset2", flameThrowerOffset)

        const playerVelocity = player.getVelocity()
        molang.setFloat("variable.dx", playerVelocity.x);
        molang.setFloat("variable.dy", playerVelocity.y);
        molang.setFloat("variable.dz", playerVelocity.z);

        molang.setFloat("variable.dspeed", 1);

        molang.setFloat("variable.prate", 50);

        molang.setFloat("variable.staylife1", 0.01);
        molang.setFloat("variable.staylife2", 3);

        if (playerVelocity.x > 0 || playerVelocity.y > 0 || playerVelocity.z > 0) {
            molang.setFloat("variable.prate", 300);

            molang.setFloat("variable.staylife1", 6);
            molang.setFloat("variable.staylife2", 12);
        }
        try {
            dimenstion.spawnParticle(particleId, player.location, molang)
        } catch (e) {
        }

    });
}

export function invertVector(vector) {
    return {
        x: -vector.x,
        y: -vector.y,
        z: -vector.z
    };
}

export function getFlameThrowerLocationForParticles(location, viewDirection) {
        const forwardOffset = 0.2;
        const rightOffset = 0.5;

        const rightDirection = {
            x: -viewDirection.z,
            y: 0,
            z: viewDirection.x
        };

        return {
            x: location.x + viewDirection.x * forwardOffset + rightDirection.x * rightOffset,
            y: location.y - viewDirection.y * forwardOffset + 1,
            z: location.z + viewDirection.z * forwardOffset + rightDirection.z * rightOffset
        };
    }

export function getLocationForParticles(location, viewDirection) {
    const offset = 0.5;
    return {
        x: location.x - viewDirection.x * offset,
        y: location.y - viewDirection.y * offset,
        z: location.z - viewDirection.z * offset
    };
}

export function calculateOffset(location1, location2) {
    return {
        x: location2.x - location1.x,
        y: location2.y - location1.y,
        z: location2.z - location1.z
    };
}